%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function compare_trianges( a)
%
%          demo function to show triangles' distance on 3D space
%          input 'a' is a Nx3 matrix with N triangles of 3 features
%          features can be anything, i.e. the length of the sides
%          
%          comparison is made for all possible combinations
%          that are n! / 2!(n-2)!
%

%
% Copyright  11-Mar-2012 George P. Pavlidis (sepdek@sepdek.net)
%
function varargout = compare_trianges( a)

% help variables
triangles = length( a(:,1));		% number of triangles to process
mxx=max(a(:,1));								% X, Y, Z min and max values
mnx=min(a(:,1));
mxy=max(a(:,2));
mny=min(a(:,2));
mxz=max(a(:,3));
mnz=min(a(:,3));

comb = fact(triangles)/(2*fact(triangles-2));		% compute number of combinations
% construct a combination index matrix
comb_mat = [];
i = 1;
while i < comb
	k = i;
	while k < triangles
		comb_mat( end+1,: ) = [i, k+1 ];
		k = k + 1;
	end
	i = i + 1;
end

d = zeros( comb, 1);			% initialize distances matrix
c = zeros( comb, 1);			% initialize cosines matrix

% compute euclidean distances and angle cosines for all combinations
for i = 1: comb,
	%d(i) = sqrt( sum( (a( comb_mat(i,1),:)-a(comb_mat(i,2),:)).^2));
	%c(i) = sum( a(comb_mat(i,1),:).*a(comb_mat(i,2),:)) / ( sqrt(sum(a(comb_mat(i,1),:).^2))*sqrt(sum(a(comb_mat(i,2),:).^2)));
	d(i) = distance( a( comb_mat(i,1),:)', a(comb_mat(i,2),:)');
	c(i) = angle( a( comb_mat(i,1),:)', a(comb_mat(i,2),:)');
end;

% report distances and angles
fprintf( '\n********** COMPUTING ALL COMBINATIONS DISTANCES AND ANGLES\n');
for i = 1: comb
	fprintf( '>>> Checking combination #%d: #%d-#%d (%.3f, %.3f, %.3f)-(%.3f, %.3f, %.3f)\n', ...
						i, comb_mat(i,1), comb_mat(i,2), a(comb_mat(i,1),:), a(comb_mat(i,2),:));
	fprintf( '    Euclidean Distance is %.3f and cosine is %.3f (angle %.3f deg.)\n', ...
						d(i),c(i),rad2deg(acos(c(i))));
	if ((d(i)==0) && (rad2deg(acos(c(i)))<1e-5))
		fprintf( '    *** Triangles are EQUAL.\n');
	end
	if ((d(i)~=0) && (rad2deg(acos(c(i)))<1e-5))
		fprintf( '    *** Triangles are SIMILAR.\n');
	end;
end;
fprintf('*********************************\n');

switch nargout
	case 1
		varargout{1} = d;
	case 2
		varargout{1} = d;
		varargout{2} = rad2deg( acos( c));
	case 3
		varargout{1} = d;
		varargout{2} = rad2deg( acos( c));
		varargout{3} = comb_mat;
end

% plot the feature space
plot3(a(:,1),a(:,2),a(:,3),'o'); 
for i = 1:triangles
	text(a(i,1)-0.1*rand,a(i,2)-0.1*rand,a(i,3)-0.1*rand,sprintf('%d',i)); 
end;
xlabel('x'); ylabel('y'); zlabel('z');
axis([min(0,mnx) mxx min(0,mny) mxy min(0,mnz) mxz]); 
grid on; rotate3d on; 
