%
% y = scale( x[, initial_interval], final_interval );
%
%     scale values of X to final_interval
%     intervals are 1x2 arrays of the form [lo_limit,hi_limit]
%     if initial_interval is ommited the function calls LIMITS function
%
% See also LIMITS
%

% (c) 7-May-2001 George P. Pavlidis
%
function y = scale( x, varargin );

	if nargin<2,
		error([upper(mfilename),':: Syntax: y=',mfilename,'(x[,in_range],out_range)']);
	end;

	if nargin==2,
		inrange=limits(x);
		outrange=varargin{1};
	else
		inrange=varargin{1};
		outrange=varargin{2};
	end;

	if diff(inrange)==0, %% row or column vector matrix or scalar
		%% just do a clipping...
		if x>=outrange(2), y=outrange(2); elseif x<=outrange(1), y=outrange(1); else y=x; end;
	else
		%% actually scale the data
		y = outrange(1) + (x-inrange(1))*abs(diff(outrange))/abs(diff(inrange));
	end;

	return;

	
%%%%%%%%%% older version %%% had some problems too!!!	
	
	
	inrangelo=inrange(1);
	inrangehi=inrange(2);
	outrangelo=outrange(1);
	outrangehi=outrange(2);
	if inrangelo>inrangehi | outrangelo>outrangehi,
		error([upper(mfilename),':: low bound should be lower than high bound in an interval!']);
	end;

	inoffs=0;
	outoffs=0;
	if outrangehi<0, outrangehi=-outrangehi; outrangelo=-outrangelo; outoffs=-outrangehi-outrangelo-1; end;
	if inrangehi<0, inrangehi=-inrangehi; inrangelo=-inrangelo; inoffs=-inrangehi-inrangelo-1; end;
	if outrangelo<0, outrangehi=outrangehi-outrangelo; outoffs=outrangelo; outrangelo=0; end;
	if inrangelo<0, inrangehi=inrangehi-inrangelo; inoffs=inrangelo; inrangelo=0; end;

	y=((((x-inoffs)-inrangelo)*outrangehi/inrangehi)+outrangelo)+outoffs;

return;